import re


def convert_env_list(src_env_class, dst_env_class, src_env_names):
    def get_dst_env(src_env_name):
        return dst_env_class(src_env_class(src_env_name).executable)

    dst_envs = map(get_dst_env, src_env_names)

    return [dst_env.name for dst_env in dst_envs if dst_env.name]


class AbstractPythonEnv(object):
    """Abstraction of list of Pythons to run against -- e.g.:
       - Travis: 2.5, 2.6, 2.7, 3.2, pypy
       - Tox: py25, py26, py27, py32, pypy, jython

    """

    executable = ''

    def __init__(self, s):
        pass

    @property
    def name(self):
        pass


class TravisPythonEnv(AbstractPythonEnv):

    regex = '(?P<major_version>\d)\.(?P<minor_version>\d)'

    def __init__(self, s):
        match = re.match(self.regex, s)

        if match:
            group_dict = match.groupdict()
            major_version = int(group_dict['major_version'])
            minor_version = int(group_dict['minor_version'])
            self.executable = 'python%d.%d' % (major_version, minor_version)
        else:
            self.executable = s

    @property
    def name(self):
        if self.executable in ('python2.5', 'python2.6', 'python2.7', 'python3.2', 'pypy'):
            return self.executable.replace('python', '')


class ToxPythonEnv(AbstractPythonEnv):

    regex = 'py(?P<major_version>\d)(?P<minor_version>\d)'

    def __init__(self, s):
        match = re.match(self.regex, s)

        if match:
            group_dict = match.groupdict()
            self.major_version = int(group_dict['major_version'])
            self.minor_version = int(group_dict['minor_version'])
            self.executable = 'python%d.%d' % (self.major_version, self.minor_version)
        else:
            self.executable = s

    @property
    def name(self):
        if self.executable in ('pypy', 'jython'):
            return self.executable
        else:
            regex = 'python(?P<major_version>\d)\.(?P<minor_version>\d)'
            match = re.match(regex, self.executable)

            if match:
                group_dict = match.groupdict()
                major_version = int(group_dict['major_version'])
                minor_version = int(group_dict['minor_version'])

            return 'py%d%d' % (major_version, minor_version)


class EnvList(object):
    """Abstraction of list of Pythons to run against -- e.g.:
       - Travis: 2.5, 2.6, 2.7, 3.2, pypy
       - Tox: py25, py26, py27, py32, pypy, jython

    """
    pass


class CIConfig(object):

    @classmethod
    def from_file(cls, f):
        pass

    def get_envlist(self):
        """Return an ``Envlist``?"""
        pass

    def get_commands(self):
        pass


class TravisCIConfig(CIConfig):
    pass

class ToxCIConfig(CIConfig):
    pass



"""Utilities for converting from Travis CI configs to Tox configs."""

from .travisconfig import TravisConfig
from .toxconfig import ToxConfig


def travis_env_to_tox_env(travis_env):
    """Converts a Travis-style environment (e.g.: "2.6") to a Tox-style
    environment (e.g.: "py26")."""

    travis_env = str(travis_env)

    if not travis_env.startswith('py') and travis_env[1] == '.':
        return 'py' + travis_env.replace('.', '')
    else:
        return travis_env


def tox_envlist_to_travis_envlist(tox_envlist):
    """Converts Tox-style environments (e.g.: "py26") to a Travis-style
    environments (e.g.: "2.6")."""

    travis_envlist = []

    for env in tox_envlist:
        if env.startswith('py'):
            if env == 'pypy':
                travis_envlist.append(env)
            else:
                travis_envlist.append(env[2] + '.' + env[3])

    return travis_envlist


def travis2tox(in_file):
    """Takes a path or file object for a ``.travis.yml`` file and returns a
    ``ToxConfig`` object."""

    config = TravisConfig(in_file)
    envlist = map(travis_env_to_tox_env, config.python)
    commands = config.get_all_commands()

    return ToxConfig(envlist, commands)


def tox2travis(in_file):
    config = ToxConfig.from_file(in_file)
    travis_config = TravisConfig()
    travis_config.language = 'python'
    travis_config.python = tox_envlist_to_travis_envlist(config.envlist)
    travis_config.script = config.commands

    return travis_config.dumps()



#!/usr/bin/env python

"""
Program for converting between different CI config files, like `.travis.yml`
and `tox.ini`.

"""

import argparse

from panci.travis2tox import travis2tox
from panci.tox2travis import tox2travis


def main():
    """The ``main`` for the ``panci`` program."""

    parser = argparse.ArgumentParser(
        description='Convert a .travis.yml file to a tox.ini file or vice-versa.')
    parser.add_argument(
        '-t', '-w', '--to', '--write',
        metavar='FORMAT',
        help='output format')
    parser.add_argument(
        'input_file',
        help='path to a .travis.yml or tox.ini file')
    args = parser.parse_args()

    if args.to == 'tox' or args.input_file.endswith('.yml'):
        tox_config = travis2tox(args.input_file)
        print(tox_config.getvalue())
    elif args.to == 'travis' or args.input_file.endswith('.ini'):
        travis_config = tox2travis(args.input_file)
        print(travis_config)


if __name__ == '__main__':
    main()



import unittest

from panci.ciconfig import TravisPythonEnv, ToxPythonEnv, convert_env_list


class ToxPythonEnvTests(unittest.TestCase):

    def test_py24(self):
        env = ToxPythonEnv('py24')
        self.assertEqual(env.executable, 'python2.4')
        self.assertEqual(env.name, 'py24')

    def test_py25(self):
        env = ToxPythonEnv('py25')
        self.assertEqual(env.executable, 'python2.5')
        self.assertEqual(env.name, 'py25')

    def test_py26(self):
        env = ToxPythonEnv('py26')
        self.assertEqual(env.executable, 'python2.6')
        self.assertEqual(env.name, 'py26')

    def test_py27(self):
        env = ToxPythonEnv('py27')
        self.assertEqual(env.executable, 'python2.7')
        self.assertEqual(env.name, 'py27')

    def test_py30(self):
        env = ToxPythonEnv('py30')
        self.assertEqual(env.executable, 'python3.0')
        self.assertEqual(env.name, 'py30')

    def test_py31(self):
        env = ToxPythonEnv('py31')
        self.assertEqual(env.executable, 'python3.1')
        self.assertEqual(env.name, 'py31')

    def test_py32(self):
        env = ToxPythonEnv('py32')
        self.assertEqual(env.executable, 'python3.2')
        self.assertEqual(env.name, 'py32')

    def test_pypy(self):
        env = ToxPythonEnv('pypy')
        self.assertEqual(env.executable, 'pypy')
        self.assertEqual(env.name, 'pypy')

    def test_jython(self):
        env = ToxPythonEnv('jython')
        self.assertEqual(env.executable, 'jython')
        self.assertEqual(env.name, 'jython')


class TravisPythonEnvTests(unittest.TestCase):

    def test_py25(self):
        env = TravisPythonEnv('2.5')
        self.assertEqual(env.executable, 'python2.5')
        self.assertEqual(env.name, '2.5')

    def test_py26(self):
        env = TravisPythonEnv('2.6')
        self.assertEqual(env.executable, 'python2.6')
        self.assertEqual(env.name, '2.6')

    def test_py27(self):
        env = TravisPythonEnv('2.7')
        self.assertEqual(env.executable, 'python2.7')
        self.assertEqual(env.name, '2.7')

    def test_py32(self):
        env = TravisPythonEnv('3.2')
        self.assertEqual(env.executable, 'python3.2')
        self.assertEqual(env.name, '3.2')

    def test_pypy(self):
        env = TravisPythonEnv('pypy')
        self.assertEqual(env.executable, 'pypy')
        self.assertEqual(env.name, 'pypy')


class ToxToTravisPythonEnvTests(unittest.TestCase):

    def test_py25(self):
        env = TravisPythonEnv(ToxPythonEnv('py25').executable)
        self.assertEqual(env.executable, 'python2.5')
        self.assertEqual(env.name, '2.5')

    def test_py26(self):
        env = TravisPythonEnv(ToxPythonEnv('py26').executable)
        self.assertEqual(env.executable, 'python2.6')
        self.assertEqual(env.name, '2.6')

    def test_py27(self):
        env = TravisPythonEnv(ToxPythonEnv('py27').executable)
        self.assertEqual(env.executable, 'python2.7')
        self.assertEqual(env.name, '2.7')

    def test_py32(self):
        env = TravisPythonEnv(ToxPythonEnv('py32').executable)
        self.assertEqual(env.executable, 'python3.2')
        self.assertEqual(env.name, '3.2')

    def test_pypy(self):
        env = TravisPythonEnv(ToxPythonEnv('pypy').executable)
        self.assertEqual(env.executable, 'pypy')
        self.assertEqual(env.name, 'pypy')

    def test_tox_to_travis_list_1(self):
        tox_env_list = ['py24', 'py25', 'py26', 'py27', 'py30', 'py31', 'py32', 'pypy', 'jython']
        expected_travis_env_list = ['2.5', '2.6', '2.7', '3.2', 'pypy']
        travis_env_list = convert_env_list(ToxPythonEnv, TravisPythonEnv, tox_env_list)
        self.assertEqual(travis_env_list, expected_travis_env_list)

    def test_tox_to_travis_list_2(self):
        tox_env_list = ['py26', 'py27', 'py32', 'pypy', 'jython']
        expected_travis_env_list = ['2.6', '2.7', '3.2', 'pypy']
        travis_env_list = convert_env_list(ToxPythonEnv, TravisPythonEnv, tox_env_list)
        self.assertEqual(travis_env_list, expected_travis_env_list)

    def test_tox_to_travis_list_3(self):
        tox_env_list = ['py24', 'pypy', 'jython']
        expected_travis_env_list = ['pypy']
        travis_env_list = convert_env_list(ToxPythonEnv, TravisPythonEnv, tox_env_list)
        self.assertEqual(travis_env_list, expected_travis_env_list)

    def test_tox_to_travis_list_4(self):
        tox_env_list = ['py24', 'jython', 'abc']
        expected_travis_env_list = []
        travis_env_list = convert_env_list(ToxPythonEnv, TravisPythonEnv, tox_env_list)
        self.assertEqual(travis_env_list, expected_travis_env_list)

    def test_tox_to_travis_list_5(self):
        tox_env_list = ['']
        expected_travis_env_list = []
        travis_env_list = convert_env_list(ToxPythonEnv, TravisPythonEnv, tox_env_list)
        self.assertEqual(travis_env_list, expected_travis_env_list)

    def test_travis_to_tox_list_1(self):
        travis_env_list = ['2.5', '2.6', '2.7', '3.2', 'pypy']
        expected_tox_env_list = ['py25', 'py26', 'py27', 'py32', 'pypy']
        tox_env_list = convert_env_list(TravisPythonEnv, ToxPythonEnv, travis_env_list)
        self.assertEqual(tox_env_list, expected_tox_env_list)

    def test_travis_to_tox_list_2(self):
        travis_env_list = ['2.5', '2.6', '2.7', '3.2', '3.3', 'pypy']
        expected_tox_env_list = ['py25', 'py26', 'py27', 'py32', 'py33', 'pypy']
        tox_env_list = convert_env_list(TravisPythonEnv, ToxPythonEnv, travis_env_list)
        self.assertEqual(tox_env_list, expected_tox_env_list)

    def test_travis_to_tox_list_3(self):
        travis_env_list = []
        expected_tox_env_list = []
        tox_env_list = convert_env_list(TravisPythonEnv, ToxPythonEnv, travis_env_list)
        self.assertEqual(tox_env_list, expected_tox_env_list)


class TravisToToxPythonEnvTests(unittest.TestCase):

    def test_py25(self):
        env = ToxPythonEnv(TravisPythonEnv('2.5').executable)
        self.assertEqual(env.executable, 'python2.5')
        self.assertEqual(env.name, 'py25')

    def test_py26(self):
        env = ToxPythonEnv(TravisPythonEnv('2.6').executable)
        self.assertEqual(env.executable, 'python2.6')
        self.assertEqual(env.name, 'py26')

    def test_py27(self):
        env = ToxPythonEnv(TravisPythonEnv('2.7').executable)
        self.assertEqual(env.executable, 'python2.7')
        self.assertEqual(env.name, 'py27')

    def test_py32(self):
        env = ToxPythonEnv(TravisPythonEnv('3.2').executable)
        self.assertEqual(env.executable, 'python3.2')
        self.assertEqual(env.name, 'py32')

    def test_pypy(self):
        env = ToxPythonEnv(TravisPythonEnv('pypy').executable)
        self.assertEqual(env.executable, 'pypy')
        self.assertEqual(env.name, 'pypy')




"""Provides a ``TravisConfig`` class for operating on ``.travis.yml`` files"""

import yaml


def listify(string_or_list):
    """Takes a string or a list and converts strings to one item lists"""

    if hasattr(string_or_list, 'startswith'):
        return [string_or_list]
    else:
        return string_or_list


class TravisConfig(object):
    """Class for operating on ``.travis.yml`` files"""

    def __init__(self, in_file=None):
        """Create an object from a file object or filename referencing a
        ``.travis.yml`` file."""

        if in_file:
            if not hasattr(in_file, 'read'):
                in_file = open(in_file, 'r')

            self.__dict__ = yaml.load(in_file)

    def get_all_commands(self):
        """Return a list of all the commands in all of the fields that can
        contain commands (e.g.: 'before_install', 'install', 'after_install',
        'before_script', 'script', 'after_script')."""

        commands = []

        for key in ['before_install', 'install', 'after_install',
                    'before_script', 'script', 'after_script']:
            commands.extend(listify(getattr(self, key, [])))

        return commands

    def dumps(self):
        return yaml.dump(self.__dict__, default_flow_style=False, indent=2)



"""Provides a ``ToxConfig`` class for operating on ``tox.ini`` files"""

try:
    # Python 2
    from ConfigParser import ConfigParser
except ImportError:
    # Python 3
    from configparser import ConfigParser

from string import Template


class ToxConfig(object):
    """Class for operating on ``tox.ini`` files"""

    @classmethod
    def from_file(self, in_file):
        if not hasattr(in_file, 'read'):
            in_file = open(in_file, 'r')

        config_parser = ConfigParser()

        try:
            # Python 3
            config_parser.read_file(in_file)
        except AttributeError:
            # Python 2
            config_parser.readfp(in_file)

        return ToxConfig(
            envlist=list(map(str.strip, config_parser.get('tox', 'envlist').split(','))),
            commands=config_parser.get('testenv', 'commands')
        )


    def __init__(self, envlist, commands):
        """Create an object from a list of environments and a list of
        commands."""

        self.envlist = envlist
        self.commands = commands

    def getvalue(self):
        """Return a string with the contents of a ``tox.ini`` file."""

        envlist = ', '.join(self.envlist)
        commands = self.get_commands()

        return Template("""
# Tox (http://tox.testrun.org/) is a tool for running tests
# in multiple virtualenvs. This configuration file will run the
# test suite on all supported python versions. To use it, "pip install tox"
# and then run "tox" from this directory.

[tox]
envlist = $envlist

[testenv]
commands = $commands
        """).substitute(
            envlist=envlist,
            commands=commands,
        ).lstrip()

    def get_commands(self):
        """Return a string with the content of the ``commands`` key for a
        ``tox.ini`` file."""

        commands = []

        if hasattr(self.commands, 'startswith'):
            commands.append(self.commands)
        elif hasattr(self.commands, '__getitem__'):
            commands.extend(self.commands)

        if len(commands) == 1:
            return commands[0]
        elif len(commands) > 1:
            return "\n    " + "\n    ".join(commands)



import os
import sys
from setuptools import setup, find_packages

install_requires = ['PyYAML']

version = sys.version_info[:2]

if version < (2,7) or (3,0) <= version <= (3,1):
    install_requires += ['argparse']

this_dir = os.path.dirname(__file__)
long_description = "\n" + open(os.path.join(this_dir, 'README.rst')).read()

setup(
    name='panci',
    version='0.0.0',
    description=('Convert between Travis-CI `.travis.yml` files and Tox `tox.ini` files'),
    long_description=long_description,
    keywords='tox, travis, continuous integration, CI',
    author='Marc Abramowitz',
    author_email='marc@marc-abramowitz.com',
    url='https://github.com/msabramo/panci',
    packages=find_packages(),
    zip_safe=False,
    install_requires=install_requires,
    test_suite='panci.tests',
    entry_points = """\
      [console_scripts]
      panci = panci.cli:main
      panci-tox-quickstart = panci.toxquickstart:main
    """,
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Testing',
        'Natural Language :: English',
        'Intended Audience :: Developers',
    ],
)
